#!/usr/bin/env bash
PID=$$
FIFO=/tmp/FIFO${PID}
mkfifo ${FIFO}
######################################################################################################

#####################
##### FONCTIONS #####
#####################
# Calcul le temps écoulé entre 2 lancements
function calcul_temps { t=$(($2-$1)) temps=$(printf "%02dh%02dm%02ds\n" $(($t/3600)) $((($t%3600)/60)) $(($t%60))); }

# Vérfie que le disque contient assez de place pour travailler
function verif_place
{
[[ ! -d "${G2S_dossier_save}" || ! -e "${G2S_fichier_mkv}" ]] && return

# Vérification de la taille dispo
place=$(df -P "${G2S_dossier_save}" | awk 'NR==2{print $4}')

# Vérification de la taille du fichier
taille_fichier=$(du "${G2S_fichier_mkv}") taille_fichier=${taille_fichier%%$'\t'*}

# Debug
echo -e "\n>>>>>>>>>>>>>>>> Information sur la place <<<<<<<<<<<<<<<<\n
Place disponnible sur le disque dur : ${place} ko
Taille du fichier mkv : ${taille_fichier} ko" >> "${log}"

# Comparaison entre les 2 tailles, necessite environ 2 fois la taille du mkv d'origine
taille_fichier=$((${taille_fichier}*2))

if (( ${place} < ${taille_fichier} ))
then
	# Envoie des informations
	echo "SET@_progressbar.set_text(''' $(eval_gettext $'Caution: Low disk space, recommanded to have the double of mkv file size avaiable') ''')"

	# Debug
	echo -e "\nPas beaucoup de place, moins de 2 fois la taille du fichier mkv." >> "${log}"

else
    # Si tout est ok on affiche le nom du film, utile dans le cas ou il y avait un message de peu de place avant changement du dossier de sortie
    [[ ${titre} ]] && echo "SET@_progressbar.set_text(''' $(eval_gettext $'Opening:') ${titre} ''')"
fi

# Debug
echo -e "\n<<<<<<<<<<<<<<<< Information sur la place >>>>>>>>>>>>>>>>\n" >> "${log}"
}

function _dts_ac3 { dts_ac3=${1}; echo "TREE@@HIZO@@liste_pistes"; }

function _encapsulage { encapsulage=${1}; }

function _vob_srt { vob_srt=${1}; }

function _auto_save
{
if [[ ${1} == True ]]
then
	G2S_auto_save=True

	if [[ ${G2S_fichier_mkv} ]]
	then
		G2S_dossier_save="${G2S_fichier_mkv%/*}"

        if [[ -d "${G2S_dossier_save}" ]]
        then
            echo "SET@_dossier_save.set_icon_from_stock(1, gtk.STOCK_YES)"
            echo "SET@_dossier_save_tmp.set_filename('''${G2S_dossier_save}''')"
            echo "SET@_dossier_save.set_text('${G2S_dossier_save}')"
        fi
    fi

else
	G2S_auto_save=False
fi
}

function _auto_file { [[ ${1} == True ]] && G2S_auto_file=True || G2S_auto_file=False ; }

function bar_stop
{
# 1 : Valeur de la barre à la fin : 0 ou 100
# 2 : texte à envoyer dans la barre

# creation du fichier debloquant
> "${tmp}/bar_stop"

# Attente pour être sur que les boucles ont bien trouvé le fichier
sleep 0.3

# En fonction de la valeur donnée en argument, on affiche 100% ou 0%
[[ ${1} == 1 ]] && echo "SET@_progressbar.set_fraction(${1})" || echo "SET@_progressbar.set_fraction(0)"

# S'il y a un argument, on envoie le texte
[[ ${2} ]] && echo "SET@_progressbar.set_text(''' ${2} ''')"

# Suppression du fichier de pourcentage qui peut foutre le bordel...
[[ -f "${tmp}/bar_pourcentage" ]] && rm "${tmp}/bar_pourcentage"

# Suppression du fichier debloquant
rm "${tmp}/bar_stop"
}

function bar_pulse
{
# Affichage de texte dans la barre
[[ ${1} ]] && echo "SET@_progressbar.set_text(''' ${1} ''')"

# La barre va pulser tant que le fichier debloquant n'existe pas
while [[ ! -f "${tmp}/bar_stop" ]]
do
	# Deplace la barre d'un bloc
	echo 'SET@_progressbar.pulse()'

	# Mini sleep
	sleep 0.2
done &
}

function bar_pourcentage
{
# Affichage de texte dans la barre
[[ ${1} ]] && echo "SET@_progressbar.set_text(''' ${1} ''')"

# La barre va progresser tant que le fichier debloquant n'existe pas
while [[ ! -f "${tmp}/bar_stop" ]]
do
	# Si le fichier n'existe pas encore, un fait un tour
	[[ ! -f "${tmp}/bar_pourcentage" ]] && continue

	# Récupération du pourcentage
	pourcentage=$(< "${tmp}/bar_pourcentage")

	# Saut de la boucle si la valeur est identique à la precedante ou si ce ne sont pas des chiffres
	[[ ${pourcentage} != [0-9]* || ${old_pourcentage} == ${pourcentage} ]] && { sleep 0.2; continue; }

	# progression de la barre
	echo "SET@_progressbar.set_fraction($(echo "scale=2;${pourcentage}/100" | bc))"

	old_pourcentage=${pourcentage}

	sleep 0.2
done &
}

function terminal
{
# $1 : on ou off

if [[ ${1} == on ]]
then
	# Inverse la visibilité de ces 2 widgets : cache le bouton extract et affiche celui d'annulation
	echo 'TOGGLE@@VISIBLE@@_bouton_stop,_bouton_extract'

	# Grise les widgets
	echo "MULTI@@SET@@set_sensitive(False)@@_dossier_save,_fichier_mkv,_auto_file,_auto_save,_reparer_mkv,_verifier_mkv,_vbox4,_verifier_mkv,_reparer_mkv"

elif [[ ${1} == off ]]
then
	# Inverse la visibilité de ces 2 widgets : cache le bouton d'annulatione t affiche celui d'estraction
	echo 'TOGGLE@@VISIBLE@@_bouton_stop,_bouton_extract'

	# Dégrise les widgets
	echo "MULTI@@SET@@set_sensitive(True)@@_dossier_save,_fichier_mkv,_auto_file,_auto_save,_reparer_mkv,_verifier_mkv,_vbox4,_verifier_mkv,_reparer_mkv"
	[[ ${mkvalidator} == True ]] && echo 'SET@_verifier_mkv.set_sensitive(True)'
	[[ ${mkclean} == True ]] && echo 'SET@_reparer_mkv.set_sensitive(True)'

	# Suppression du fichier indiquant un travail en cours
	[[ -f "${tmp}/action_en_cours" ]] && rm "${tmp}/action_en_cours"

	# Suppression du fichier listant les fichiers pouvant être supprimé en cas d'arret
	[[ -f "${tmp}/fichiers" ]] && rm "${tmp}/fichiers"
fi
}


##################################################
##### ETAPE 1 : SELECTION DU FICHIER/DOSSIER #####
##################################################
# Récupération temporaire du dossier de sortie, sera ou non utilisé en fonction du bouton pressé
function _dossier_save_tmp { dossier_save_tmp="${@}"; }

function dossier_save_ok
{
# Le choix du fichier mkv est validé, envoie de l'adresse à la boite d'entry
echo "SET@_dossier_save.set_text('${dossier_save_tmp}')"

# Execute la fonction _dossier_save avec l'adresse du dossier
_dossier_save "${dossier_save_tmp}"
}

function _dossier_save
{
# Si nouvelle adresse identique à l'ancienne on arrete là
[[ "${@}" == "${G2S_dossier_save}" ]] && return

# Validation de l'adresse
if [[ ! -d "${@}" ]]
then
    echo "SET@_dossier_save.set_icon_from_stock(1, gtk.STOCK_NO)"

else
    echo "SET@_dossier_save.set_icon_from_stock(1, gtk.STOCK_YES)"

    # Envoie de l'adresse dans la box de sélection
    echo "SET@_dossier_save_tmp.set_filename('${@}')"

    # Récupération de l'adresse
    G2S_dossier_save=${@}

    # Vérification de la place
    verif_place
fi
}


# Récupération temporaire du nom du fichier mkv, sera ou non utilisé en fonction du bouton pressé
function _fichier_mkv_tmp
{
if [[ ${1} == my_callback ]]
then
    # Dans le cas d'un double clic directement sur le fichier choisi
    fichier_mkv_ok
else
    # Dans le cas d'un clic ou du premier clic sur le fichier
    fichier_mkv_tmp="${@}"
fi
}

function fichier_mkv_ok
{
# Le choix du fichier mkv est validé, envoie de l'adresse à la boite d'entry
echo "SET@_fichier_mkv.set_text('${fichier_mkv_tmp}')"

# Execute la fonction _fichier_mkv avec l'adresse du fichier mkv
_fichier_mkv "${fichier_mkv_tmp}"
}


function _fichier_mkv
{
# En cas de drop ou d'adresse commencant par file://
if [[ ${@} =~ ^file: ]]
then
	# En cas de drop, gnome fait des truc zarb avec les espaces en autre
	fichier=$(echo -e "${@//\%/\\x}") fichier="${fichier#*file:??}"

	# Si nouvelle adresse identique à l'ancienne on arrete là
	[[ "${fichier}" == "${G2S_fichier_mkv}" ]] && return

    # Validation de l'adresse
    if [[ ! -f "${fichier}" ]]
    then
        echo "SET@_fichier_mkv.set_icon_from_stock(1, gtk.STOCK_NO)"
        return
    else
        echo "SET@_fichier_mkv.set_icon_from_stock(1, gtk.STOCK_YES)"
    fi

    # Mise à jour de la boite d'entry
    echo "SET@_fichier_mkv.set_text('${fichier}')"

    # Récupération du nom du fichier mkv et de son adresse
    G2S_fichier_mkv="${fichier}" nom_fichier=${G2S_fichier_mkv##*/}

else
    # Si nouvelle adresse identique à l'ancienne on arrete là
    #[[ "${@}" == "${G2S_fichier_mkv}" ]] && return

    # Validation de l'adresse
    if [[ ! -f "${@}" ]]
    then
        echo "SET@_fichier_mkv.set_icon_from_stock(1, gtk.STOCK_NO)"
        return
    else
        echo "SET@_fichier_mkv.set_icon_from_stock(1, gtk.STOCK_YES)"
    fi

    # Récupération du nom du fichier mkv et de son adresse
    G2S_fichier_mkv="${@}" nom_fichier=${G2S_fichier_mkv##*/}
fi

# Envoie de l'adresse dans la box de sélection
echo "SET@_fichier_mkv_tmp.set_filename('${G2S_fichier_mkv}')"

# Aller retour de la barre pendant le chargement des infos
bar_pulse

# Modifications graphiques
[[ ${G2S_auto_save} == True ]] && echo "SET@_dossier_save_tmp.set_current_folder('${G2S_fichier_mkv%/*}')"
echo 'SET@store_liste_pistes.clear()'
echo 'SET@liste_pistes.columns_autosize()'
echo 'MULTI@@SET@@set_active(False)@@_vob_srt,_dts_ac3,_encapsulage'
echo "MULTI@@SET@@set_sensitive(False)@@_dossier_save,_fichier_mkv,_auto_file,_auto_save,_reparer_mkv,_verifier_mkv,_vbox4,_verifier_mkv,_reparer_mkv,_vob_srt,_mkvinfo,_encapsulage"

# obligatoire car le set_active(False) n'execute pas les toggle
vob_srt=False dts_ac3=False encapsulage=False

# Suppression du contenu du dossier temporaire s'il existe
[[ ! -d "${tmp}" ]] && mkdir -p "${tmp}"
[[ $(ls "${tmp}") ]] && rm "${tmp}"/* &

[[ ${mkvalidator} == True ]] && echo 'SET@_verifier_mkv.set_sensitive(True)'
[[ ${mkclean} == True ]] && echo 'SET@_reparer_mkv.set_sensitive(True)'
[[ ${ffmpeg} == True ]] && echo 'SET@_dts_ac3.set_sensitive(True)'

# Récupération des infos du fichier
mkv_mkvmerge=$(export LANGUAGE=en; mkvmerge -I "${G2S_fichier_mkv}" | sed "s/codec_private_data:[0-9a-zA-Z]* //")
mkv_mkvmerge=${mkv_mkvmerge//\\c/:}

# mkvinfo sur le fichier mkv
(export LANGUAGE=en; mkvinfo "${G2S_fichier_mkv}") > "${tmp}/mkvinfo"

retour_mkvmerge=${mkv_mkvmerge//$'\n'/\\n\\n} retour_mkvmerge=${retour_mkvmerge//\\s/ }

# Debug
echo -e "\n>>>>>>>>>>>>>>>> Traitement du fichier <<<<<<<<<<<<<<<<\n
-------Commande-------
export LANGUAGE=en; mkvmerge -I \"${G2S_fichier_mkv}\" | sed \"s/codec_private_data:[0-9a-zA-Z]* //\"\n
-------fichier_mkv-------
${G2S_fichier_mkv}\n
-------nom_fichier-------
${nom_fichier}\n
-------mkv_mkvmerge-------
${retour_mkvmerge}\n
<<<<<<<<<<<<<<<< Traitement du fichier >>>>>>>>>>>>>>>>\n" >> "${log}"

if [[ ${mkv_mkvmerge} =~ "container: Matroska" ]]
then
	# Lance la vérification de place
	verif_place &

	# Récupération du titre du fichier si précisé sinon utilise le nom du fichier
	titre=$(sed -n "/Title:/ s/.*Title: //p" "${tmp}/mkvinfo")
	[[ -z ${titre} ]] && titre="${nom_fichier}"

	# Si le titre existe, on l'utilise, sinon c'est le nom du fichier
	echo "SET@_progressbar.set_text(''' $(eval_gettext $'Opening:') ${titre} ''')"

	# Suppression de la 1ere ligne contenant le nom du fichier et le conteneur
	mkv_mkvmerge="${mkv_mkvmerge#*$'\n'}"

	# Lancement de la fonction scan => envoie vers l'etape 2
	scan

else
	# Debug
	echo -e "\nFichier mkv on reconnu par mkvmerge" >> "${log}"

	# Info
	echo "SET@_progressbar.set_text(''' $(eval_gettext $'Error: mkvmerge does not recognize the mkv file.') ''')"
fi

# Retour à au début des treeview
echo 'MULTI@@SET@@scroll_to_cell(0)@@liste_pistes'

# Arret de la barre et remise à 0
bar_stop 0

# Dégrisage des widgets
echo "MULTI@@SET@@set_sensitive(True)@@_dossier_save,_fichier_mkv,_auto_file,_auto_save,_reparer_mkv,_verifier_mkv,_vbox4,_verifier_mkv,_reparer_mkv,_mkvinfo"
}


###############################################
##### ETAPE 2 : CHARGEMENT DU FICHIER MKV #####
###############################################
function scan
{
echo -e "\n>>>>>>>>>>>>>>>> Scan des pistes du fichier mkv <<<<<<<<<<<<<<<<" >> "${log}"

# Boucle permettant de travailler sur la liste de track
while read track
do
	# Récupération du type de track
	type=${track#*: } type=${type%% *} autre_type=${track%%:*}

	if [[ ${type} == video || ${type} == audio || ${type} == subtitles || ${type} == type ]]
	then
		case ${type} in
			video)
				# Récupération des variables
				icone='video-x-generic'
				icone2="dialog-cancel"
				ID=${track#*ID } ID=${ID%%:*}

				codec="${track#*(}" codec="${codec%%)*}"
				case ${codec//\//_} in
					*H264*|V_MPEG4_ISO_AVC|avc1) codec="h264" ;;
					*XVID*) codec="xvid" ;;
					V_REAL_RV40) codec="rv40" ;;
					V_MPEG4_ISO_ASP) codec="mpeg4" ;;
					*) codec="${codec//\//_}" ;;
				esac

				# Récupération de la valeur si l'information track_name existe sinon utilisation de la resolution
				[[ "${track}" =~ "track_name:" ]] && info1=${track#*track_name:} info1=${info1%% *} info1=${info1//-_-/ } || info1=${track#*display_dimensions:} info1=${info1%% *}

				# Récupération des fps
				info2=$(sed -n "/Default duration/ s/.*(\([^ ]*\).*/\1/p" "${tmp}/mkvinfo" | head -1)

				# Modification du nombre de fps 23.98 en 23.976
				[[ ${info2} == "23.98 fps" ]] && info2="23.976 fps" ;;

			audio)
				# Récupération des variables
				icone='audio-x-generic'
				icone2="dialog-cancel"
				ID=${track#*ID } ID=${ID%%:*}

				codec="${track#*(}" codec="${codec%%)*}"
				case ${codec//\//_} in
					A_AAC*|mp4a|he-aac) codec="aac" ;;
					A_AC3) codec="ac3" ;;
					A_DTS) codec="dts" ;;
					A_VORBIS|Ogg) codec="ogg" ;;
					A_AC3|A_MPEG/L3) codec="mp3" ;;
					*) codec="${codec//\//_}" ;;
				esac

				# Récupération de la valeur si l'information track_name existe
				if [[ "${track}" =~ "track_name:" ]]
				then
					info1=${track#*track_name:} info1=${info1%% *} info1=${info1//-_-/ }

				# Récupération de la valeur si l'information audio_sampling_frequency existe
				elif [[ "${track}" =~ "audio_sampling_frequency:" ]]
				then
					info1=${track#*audio_sampling_frequency:} info1=${info1%% *} info1=${info1//-_-/ } info1="${info1} Hz"
				fi

				# Récupération de la valeur si l'information language existe
				if [[ "${track}" =~ "language:" ]]
				then
					info2=${track#*language:} info2=${info2%% *}

				# Récupération de la valeur si l'information audio_channels existe
				elif [[ "${track}" =~ "audio_channels:" ]]
				then
					info2=${track#*audio_channels:} info2=${info2%% *} info1=${info1//-_-/ } info1="${info1} $(eval_gettext $'channels')"
				fi ;;

			subtitles)
				# Récupération des variables
				ID=${track#*ID } ID=${ID%%:*}
				icone2="dialog-cancel"

				codec="${track#*(}" codec="${codec%%)*}"
				case ${codec//\//_} in
					S_VOBSUB|vobsub) codec="vobsub" icone='preferences-desktop-font' ;;
					text|S_TEXT_UTF8) codec="srt" icone='text-x-generic' ;;
					S_TEXT_SSA) codec="ssa" icone='text-x-generic' ;;
					*) codec="${codec//\//_}" icone='text-x-generic' ;;
				esac

				# Récupération de la valeur si l'information track_name existe
				[[ "${track}" =~ "track_name:" ]] && info1=${track#*track_name:} info1=${info1%% *} info1=${info1//-_-/ }

				# Récupération de la valeur si l'information language existe
				[[ "${track}" =~ "language:" ]] && info2=${track#*language:} info2=${info2%% *} ;;

			type)
				# Récupération des variables
				ID=${track#*ID } ID=${ID%%:*}
				type=${track#* type ?} type=${type%%/*}
				codec=${track#*/} codec=${codec%%?, *}
				info2=${track##*size } info2=${info2%%,*}
				icone2="dialog-ok"

				# Si une description existe, on l'utilise sinon, on recupere le nom
				[[ "${track}" =~ ", description " ]] && info1=${track##*, description ?} info1=${info1%%?, *} || info1=${track%?} info1=${info1##*file name ?}

				# Quelques exceptions
				case ${codec} in
					x-truetype-font) codec="font" ;;
					x-msdos-program) codec="application msdos" ;;
					plain) codec="text" ;;
					x-flac) codec="flac" ;;
					x-flv) codec="flv" ;;
					x-ms-bmp) codec="bmp" ;;
					ogg|ogm) type="media" ;; # Ils sont reconnus entant qu'applications
				esac

				# Récupération de l'icone à utiliser
				case "${type}" in
					application*) icone="system-run" ;;
					image) icone="image-x-generic" ;;
					text) icone="accessories-text-editor" ;;
					media|video|audio) icone="applications-multimedia" ;;
					web) icone="applications-internet" ;;
					*) icone="image-missing" ;;
				esac

				# Si pas d'infos, texte de base
				[[ -z ${info1} ]] && info1="$(eval_gettext $'No name')"
				[[ -z ${info2} ]] && info2="$(eval_gettext $'No information')" ;;
		esac

		# Si pas d'infos, texte de base
		[[ -z ${info1} ]] && info1="$(eval_gettext $'No name')"
		[[ -z ${info2} ]] && info2="$(eval_gettext $'No information')"

		# Envoie des infos dans le tree des pistes
		echo "TREE@@END@@liste_pistes@@False|${icone}|${ID}|${icone2}|${info1}|${info2}|${codec}"

	else
		case ${autre_type} in
			Chapters)
				# Récupération des variables
				info1=${track#*: } info1=${info1% *}
				info2=${chapters}
				icone="x-office-address-book"

				# Envoie des infos dans les tree
				echo "TREE@@END@@liste_pistes@@False|${icone}|chapters|dialog-ok|${info2}|${info1} ${info2}|Chapters" ;;

			"Global tags")
				# Récupération des variables
				info1=${track#*: } info1=${info1% *}
				info2=${tags}
				icone="text-html"

				# Envoie des infos dans les tree
				echo "TREE@@END@@liste_pistes@@False|${icone}|tags|dialog-ok|${info2}|${info1} ${info2}|Tags" ;;
		esac
	fi

# Debug
echo -e "\n-------Piste ${track}-------
type : ${type}
autre_type : ${autre_type}
ID : ${ID}
codec : ${codec}
info1 : ${info1}
info2 : ${info2}
icone : ${icone}
icone2 : ${icone2}\n" >> "${log}"

# Destruction des variables
unset type autre_type icone ID codec info1 info2

done <<< "${mkv_mkvmerge//\\s/-_-}"

echo -e "<<<<<<<<<<<<<<<< Scan des pistes du fichier mkv >>>>>>>>>>>>>>>>\n" >> "${log}"
}


#######################################################################
##### ETAPE 3 : SELECTION DES PISTES ET LANCEMENT DE L'EXTRACTION #####
#######################################################################
function liste_pistes
{
# action : Action réalisée sur le tree : toggled combo clic drop hizo edit
action=${@} action=${action%%@*}

# valeurs[0] : Etat de la case à cocher : True False
# valeurs[1] : Icone de la piste
# valeurs[2] : ID de la piste
# valeurs[3] : Icone de visualisation : dialog-ok dialog-cancel
# valeurs[4] : Description de la piste
# valeurs[5] : Codec de la piste	=> le changer en message de survol sur l'icone ?
# valeurs[6] : Information sur la piste
valeurs="${@}" valeurs="${valeurs#*@*@}"
old_IFS=${IFS} IFS='|' valeurs=(${valeurs}) IFS=${old_IFS}

case ${action} in
	# Si l'on clique sur l'icône d'état
	toggled)
		# Relance la fonction liste_pistes avec pour argument tout le contenu du tree
		echo "TREE@@HIZO@@liste_pistes" ;;


	# Si l'on clique sur l'icône de visualisation
	clic)
		# Si la visualisation est possible alors on extrait et on lance l'affichage
		if [[ ${valeurs[3]} == dialog-ok ]]
		then
			# Extraction du fichier joint s'il n'existe pas et ouverture du fichier
			case ${valeurs[1]} in
				x-office-address-book)
					if [[ ! -e "${tmp}/${valeurs[2]}" ]]
					then
						[[ ${valeurs[2]} == chapters ]] && mkvextract $valeurs[2]} "${G2S_fichier_mkv}" -s > "${tmp}/${valeurs[2]}" || mkvextract ${valeurs[2]} "${G2S_fichier_mkv}" > "${tmp}/${valeurs[2]}"
					fi

					xdg-open "${tmp}/${valeurs[2]}" ;;

				accessories-text-editor)
					[[ ! -e "${tmp}/attachments_${valeurs[2]}" ]] && mkvextract attachments "${G2S_fichier_mkv}" ${valeurs[2]}:"${tmp}/attachments_${valeurs[2]}" && iconv -f ISO-8859-1 -t UTF8 "${tmp}/attachments_${valeurs[2]}" -o "${tmp}/attachments_${valeurs[2]}"

					xdg-open "${tmp}/attachments_${valeurs[2]}" ;;

				image-x-generic|applications-internet|applications-multimedia|image-missing)
					[[ ! -e "${tmp}/attachments_${valeurs[2]}" ]] && mkvextract attachments "${G2S_fichier_mkv}" ${valeurs[2]}:"${tmp}/attachments_${valeurs[2]}"

					xdg-open "${tmp}/attachments_${valeurs[2]}" ;;

			# Il reste les icones : system-run et image-missing
			esac
		fi ;;


	# Si l'on modifie les infos des pistes
	edit)
		# Relance la fonction liste_pistes avec pour argument tout le contenu du tree
		echo "TREE@@HIZO@@liste_pistes" ;;


	# Si l'on modifie le combo
	combo)
		# Il y a une valeur de plus avant le 1er element dans le cas du combo
		# combo@1,5@30.000 fps@False|audio-x-generic|1|dialog-cancel|Japonais|jap|aac
		valeurs[0]=${valeurs[0]#*@}

		# Vérification du nombre de fps
		ligne=${@} ligne=${ligne#*@} ligne=${ligne%%,*}
		colonne=${@} colonne=${colonne#*,} colonne=${colonne%%@*}
		old_info=${@} old_info=${old_info#*@*@} old_info=${old_info%%@*}

		case "${valeurs[1]}" in
			# Dans le cas d'audio ou de sous titres
			audio-x-generic|text-x-generic|preferences-desktop-font)
				# Vérification du formatage de l'information
				if [[ ${valeurs[5]} != [a-z][a-z][a-z] ]]
				then
					echo "TREE@@FORCE_SELECT@@liste_pistes"
					echo "TREE@@CELL@@liste_pistes@@${ligne},${colonne}@@${old_info}"

					# Debug
					echo "SET@_progressbar.set_text(''' $(eval_gettext $'Action canceled: the language should be composed of three small letters') ''')"
					echo -e "\nAction annulée : la langue doit être composée de 3 lettres miniscules\n" >> "${log}"
				else
					# Relance la fonction liste_pistes avec pour argument tout le contenu du tree
					echo "TREE@@HIZO@@liste_pistes"
				fi ;;

			# Dans le cas de vidéos
			video-x-generic)
				# Vérification du formatage de l'information
				if [[ ! ${valeurs[5]/ fps} =~ ^[0-9][0-9]$|^[0-9][0-9]\.[0-9][0-9][0-9]$ ]]
				then
					echo "TREE@@FORCE_SELECT@@liste_pistes"
					echo "TREE@@CELL@@liste_pistes@@${ligne},${colonne}@@${old_info}"

					# Debug
					echo "SET@_progressbar.set_text(''' $(eval_gettext $'Action canceled: good fps format should be: XX or XX.XXX') ''')"
					echo -e "\nAction annulée : le bon format des fps doit être : XX ou XX.XXX\n" >> "${log}"
				else
					# Relance la fonction liste_pistes avec pour argument tout le contenu du tree
					echo "TREE@@HIZO@@liste_pistes"
				fi ;;

			# Dans les autres cas, on annule
			*)
				echo "TREE@@FORCE_SELECT@@liste_pistes"
				echo "TREE@@CELL@@liste_pistes@@${ligne},${colonne}@@${fps}"

				# Debug
				echo "SET@_progressbar.set_text(''' $(eval_gettext $'Action canceled: Unable to change information for this track') ''')"
				echo -e "\nAction annulée : impossible de changer l'information pour cette piste\n" >> "${log}" ;;
		esac ;;


	# Si la fonction est lancée par TREE@@HIZO@@liste_pistes
	hizo)
		# Récupération de toutes les infos disponibles dans le tree
		tree_complet=${@#hizo?} tree_total=${tree_complet//@@/$'\n'} tree_complet=$(egrep "^True" <<< "${tree_total}")

		# Si le tree retourne des pistes séléctionnée
		if [[ ${tree_complet} ]]
		then
			# Dégrise les boutons et options
			echo 'MULTI@@SET@@set_sensitive(True)@@_bouton_extract,_encapsulage'
		else
			# Grise les boutons et options
			echo 'MULTI@@SET@@set_sensitive(False)@@_bouton_extract,_encapsulage'
			echo 'MULTI@@SET@@set_active(False)@@_encapsulage,_dts_ac3,_vob_srt'
		fi

		# En cas de piste audio DTS
		if [[ ${tree_complet} =~ audio-x-generic.*dts && ${ffmpeg} == True ]]
		then
			# Dégrise le coche de la convertion audio
			echo 'SET@_dts_ac3.set_sensitive(True)'
		else
			# Grise, desactive le coche de la convertion audio
			echo 'SET@_dts_ac3.set_sensitive(False)'
			echo 'SET@_dts_ac3.set_active(False)'
		fi

		# En cas de piste audio DTS
		if [[ ${tree_complet} =~ preferences-desktop-font ]]
		then
			# Dégrise le coche de la convertion audio
			echo 'SET@_vob_srt.set_sensitive(True)'
		else
			# Grise, desactive le coche de la convertion audio
			echo 'SET@_vob_srt.set_sensitive(False)'
			echo 'SET@_vob_srt.set_active(False)'
		fi ;;
esac
}


##################################################################
##### ETAPE 4 : CREATION DES FICHIERS MKVMERGE ET MKVEXTRACT #####
##################################################################
function _bouton_extract
{
terminal on

# Sauvegarde des préférences
echo 'CONFIG@@SAVE'; sleep 0.1

# Chargement des info
echo 'CONFIG@@GET'; sleep 0.1

# Lancement de la suite
echo 'ITER@@bouton_extract_suite'
}


function bouton_extract_suite
{
# Lance la vérification de place
verif_place

# Remise à 0 des variables pour ne pas interagir entre plusiers executions
unset dts_ffmpeg mkvextract_track mkvextract_joint mkvextract_chap mkvextract_tag mkvmerge mkvmerge_ac3

# Suppression des fichiers
rm "${tmp}"/vob_srt* 2>/dev/null

case ${G2S_bitrate:-0} in
	0) G2S_bitrate=128 ;;
	1) G2S_bitrate=192 ;;
	2) G2S_bitrate=224 ;;
	3) G2S_bitrate=256 ;;
	4) G2S_bitrate=320 ;;
	5) G2S_bitrate=384 ;;
	6) G2S_bitrate=448 ;;
	7) G2S_bitrate=512 ;;
	8) G2S_bitrate=576 ;;
	9) G2S_bitrate=640 ;;
esac

case ${G2S_boost:-0} in
	0) G2S_boost=0 ;;
	1) G2S_boost=2 ;;
	2) G2S_boost=3 ;;
	3) G2S_boost=4 ;;
	4) G2S_boost=5 ;;
esac

# Récupération de l'heure en seconde pour calculer le temps écoulé.
temps_x=$(date +%s)

# Debug
echo -e "\n>>>>>>>>>>>>>>>> Création des commandes post bouton extraction <<<<<<<<<<<<<<<<\n
dts_ac3 : ${dts_ac3}
encapsulage : ${encapsulage}
vob_srt : ${vob_srt}\n" >> "${log}"

# Création des commandes avec le début des differentes commandes
if [[ ${encapsulage} == True ]]
then
	# Creation du code pour mkvmerge
	mkvmerge="mkvmerge -o \"${G2S_dossier_save}/MEG_${nom_fichier}\""

	# Récupération du titre du mkv pour l'encapsulage
	[[ ${titre} ]] && mkvmerge+=" --title \"${titre}\""

	# Creation et Ajout du fichier dans la liste des fichiers temps
	echo "${G2S_dossier_save}/MEG_${nom_fichier}" >> "${tmp}/fichiers"
fi

# Creation des codes pour l'extraction
mkvextract_track="mkvextract tracks \"${G2S_fichier_mkv}\""
mkvextract_joint="mkvextract attachments \"${G2S_fichier_mkv}\""

while read track
do
	# valeurs[0] : Etat de la case à cocher : True False
	# valeurs[1] : Icone de la piste
	# valeurs[2] : ID de la piste
	# valeurs[3] : Icone de visualisation : dialog-ok dialog-cancel
	# valeurs[4] : Info1 : Description de la piste
	# valeurs[5] : Info2 : Information sur la piste
	# valeurs[6] : Codec de la piste
	old_IFS=${IFS}		IFS='|'
	valeurs=(${track})
	IFS=${old_IFS}

	# Destruction des variables inutiles
	[[ ${valeurs[4]} == "$(eval_gettext $'No name')" ]] && unset valeurs[4]
	[[ ${valeurs[5]} == "$(eval_gettext $'No information')" ]] && unset valeurs[5]

	# Séparation des type de fichiers car commandes différentes, Récupération de l'extension
	case ${valeurs[1]} in
		video-x-generic)
			# Récupération de l'extension de la piste
			case "${valeurs[6],,}" in
				rv*) valeurs[6]="rm" ;;
				h264) valeurs[6]="mkv" ;;
				*) valeurs[6]="avi" ;;
			esac

			# Ajout du fichier dans la liste des fichiers temps
			echo "${G2S_dossier_save}/${valeurs[2]}_Video_${valeurs[4]}.${valeurs[6]}" >> "${tmp}/fichiers"

			# Code pour extraction de la piste
			mkvextract_track+=" ${valeurs[2]}:\"${G2S_dossier_save}/${valeurs[2]}_Video_${valeurs[4]}.${valeurs[6]}\""

			# Code d'encapsulage
			if [[ ${encapsulage} == True ]]
			then
				# Verifie que le nombre de fps est non vide sinon donne la valeur 25000fps
				if [[ -z "${valeurs[5]}" ]]
				then
					valeurs[5]="25.000 fps"

					# Debug
					echo "SET@_progressbar.set_text(''' $(eval_gettext $'FPS: invalid value, use 25.000 fps') ''')"
					echo -e "\nFPS : valeur invalide, utilisation de 25.000 fps\n" >> "${log}"
				fi

				[[ ${valeurs[4]} ]] && mkvmerge+=" --track-name \"0:${valeurs[4]}\""
				mkvmerge+=" --default-duration \"0:${valeurs[5]% *}fps\" --compression \"0:none\" \"${G2S_dossier_save}/${valeurs[2]}_Video_${valeurs[4]}.${valeurs[6]}\""
			fi ;;

		audio-x-generic)
			# En cas de fichier DTS avec conversion en AC3
			if [[ ${dts_ac3} == True && ${valeurs[6]} == *[Dd][Tt][Ss]* ]]
			then
				# Code pour la conversion du son
				[[ -z ${dts_ffmpeg} ]] && dts_ffmpeg="ffmpeg -y -i \"${G2S_fichier_mkv}\""

				dts_ffmpeg+=" -vn -map 0:${valeurs[2]}"
				[[ ${boost_num} -gt 1 ]] && dts_ffmpeg+=" -vol ${G2S_boost_num}"
				[[ ${G2S_bitrate} -gt 0 ]] && dts_ffmpeg+=" -ab ${G2S_bitrate}k"

				[[ ${G2S_canaux} == True ]] && dts_ffmpeg+=" -ac 2"

				dts_ffmpeg+=" -f ac3 \"${G2S_dossier_save}/${valeurs[2]}_Audio_${valeurs[6]}_${valeurs[5]}.ac3\""

				# Ajout du fichier dans la liste des fichiers temps
				echo "${G2S_dossier_save}/${valeurs[2]}_Audio_${valeurs[6]}_${valeurs[5]}.ac3" >> "${tmp}/fichiers"

				# Code d'encapsulage
				if [[ ${encapsulage} == True ]]
				then
					[[ ${valeurs[5]} ]] && mkvmerge+=" --language \"0:${valeurs[5]}\""
					mkvmerge+=" --compression \"0:none\" --track-name \"0:${valeurs[4]:-AC3 ${G2S_bitrate}k}\" \"${G2S_dossier_save}/${valeurs[2]}_Audio_${valeurs[6]}_${valeurs[5]}.ac3\""
				fi

			else
				valeurs[6]=${valeurs[6]//vorbis/ogg}

				# Ajout du fichier dans la liste des fichiers temps
				echo "${G2S_dossier_save}/${valeurs[2]}_Audio_${valeurs[6]}_${valeurs[5]}.${valeurs[6]:-inconnu}"  >> "${tmp}/fichiers"

				# Code pour extraction de la piste
				mkvextract_track+=" ${valeurs[2]}:\"${G2S_dossier_save}/${valeurs[2]}_Audio_${valeurs[6]}_${valeurs[5]}.${valeurs[6]:-inconnu}\""

				# Code d'encapsulage
				if [[ ${encapsulage} == True ]]
				then
					[[ ${valeurs[4]} ]] && mkvmerge+=" --track-name \"0:${valeurs[4]}\""
					[[ ${valeurs[5]} ]] && mkvmerge+=" --language \"0:${valeurs[5]}\""
					mkvmerge+=" --compression \"0:none\" \"${G2S_dossier_save}/${valeurs[2]}_Audio_${valeurs[6]}_${valeurs[5]}.${valeurs[6]:-inconnu}\""
				fi
			fi ;;

		preferences-desktop-font|text-x-generic)
			valeurs[6]=${valeurs[6]//*sub/sub}

			# Modification de fre en fra
			valeurs[5]=${valeurs[5]//fre/fra}

			# Ajout du fichier dans la liste des fichiers temps
			echo "${G2S_dossier_save}/${valeurs[2]}_Sous-titres_${valeurs[5]}.${valeurs[6]:-inconnu}" >> "${tmp}/fichiers"

			# Si c'est un fichier vobsub, il y a un fichier idx de créé
			[[ ${valeurs[6]} == sub ]] && echo "${G2S_dossier_save}/${valeurs[2]}_Sous-titres_${valeurs[5]}.idx" >> "${tmp}/fichiers"

			# Code pour extraction de la piste
			mkvextract_track+=" ${valeurs[2]}:\"${G2S_dossier_save}/${valeurs[2]}_Sous-titres_${valeurs[5]}.${valeurs[6]:-inconnu}\""

			# Code d'encapsulage
			if [[ ${encapsulage} == True ]]
			then
				[[ ${valeurs[4]} ]] && mkvmerge+=" --track-name \"0:${valeurs[4]}\""
				[[ ${valeurs[5]} ]] && mkvmerge+=" --language \"0:${valeurs[5]}\""
				mkvmerge+=" --compression 0:none \"${G2S_dossier_save}/${valeurs[2]}_Sous-titres_${valeurs[5]}."

				# Dans le cas d'un fichier vobsub'
				if [[ ${valeurs[6]} == sub ]]
				then
					# S'il y a conversion des fichiers sub
					if [[ ${vob_srt} == True && ${valeurs[6]} == sub ]]
					then
						mkvmerge+="srt\""
						convert_vob_srt="True"

						 # Ajout du fichier srt à la liste des fichiers
						 echo "${G2S_dossier_save}/${valeurs[2]}_Sous-titres_${valeurs[5]}.srt" >> "${tmp}/fichiers"

						# Création des commandes de conversion
						echo "mv \"${G2S_dossier_save}/${valeurs[2]}_Sous-titres_${valeurs[5]}\"* \"${tmp}\"" >> "${tmp}/vob_srt_a"
						echo "${PWD}/subp2tiff -n \"${tmp}/${valeurs[2]}_Sous-titres_${valeurs[5]}\"" >> "${tmp}/vob_srt_b"
						echo "for fichier in \"${tmp}/${valeurs[2]}_Sous-titres_${valeurs[5]}\"*.tif \"${tmp}/${valeurs[2]}_Sous-titres_${valeurs[5]}\"*.pgm; do tesseract -l ${valeurs[5]} \"\${fichier}\" \"\${fichier}\"; done" >> "${tmp}/vob_srt_c"
						echo "${PWD}/subptools -s -w -t srt -i \"${tmp}/${valeurs[2]}_Sous-titres_${valeurs[5]}.xml\" -o \"${G2S_dossier_save}/${valeurs[2]}_Sous-titres_${valeurs[5]}.srt\"" >> "${tmp}/vob_srt_d"

					else
						mkvmerge+="idx\""
					fi

				else
					mkvmerge+="${valeurs[6]:-inconnu}\""
				fi

			else
				# S'il y a conversion des fichiers sub
				if [[ ${vob_srt} == True && ${valeurs[6]} == sub ]]
				then
					convert_vob_srt="True"

					# Ajout du fichier srt à la liste des fichiers
					echo "${G2S_dossier_save}/${valeurs[2]}_Sous-titres_${valeurs[5]}.srt" >> "${tmp}/fichiers"

					# Création des commandes de conversion
					echo "mv \"${G2S_dossier_save}/${valeurs[2]}_Sous-titres_${valeurs[5]}\"* \"${tmp}\"" >> "${tmp}/vob_srt_a"
					echo "${PWD}/subp2tiff -n \"${tmp}/${valeurs[2]}_Sous-titres_${valeurs[5]}\"" >> "${tmp}/vob_srt_b"
					echo "for fichier in \"${tmp}/${valeurs[2]}_Sous-titres_${valeurs[5]}\"*.tif \"${tmp}/${valeurs[2]}_Sous-titres_${valeurs[5]}\"*.pgm; do tesseract -l ${valeurs[5]} \"\${fichier}\" \"\${fichier}\"; done" >> "${tmp}/vob_srt_c"
					echo "${PWD}/subptools -s -w -t srt -i \"${tmp}/${valeurs[2]}_Sous-titres_${valeurs[5]}.xml\" -o \"${G2S_dossier_save}/${valeurs[2]}_Sous-titres_${valeurs[5]}.srt\"" >> "${tmp}/vob_srt_d"
				fi

			fi ;;

		x-office-address-book)
			# Ajout du fichier dans la liste des fichiers temps
			echo "${G2S_dossier_save}/${chapters}.txt" >> "${tmp}/fichiers"

			# Code pour extraction des chapitres
			mkvextract_chap="mkvextract chapters \"${G2S_fichier_mkv}\" -s > \"${G2S_dossier_save}/${chapters}.txt\""

			# Code d'encapsulage
			[[ ${encapsulage} == True ]] && mkvmerge+=" --chapters \"${G2S_dossier_save}/${chapters}.txt\"" ;;

		text-html)
			# Ajout du fichier dans la liste des fichiers temps
			echo "${G2S_dossier_save}/${tags}.xml" >> "${tmp}/fichiers"

			# Code pour extraction des tags
			mkvextract_tag="mkvextract tags \"${G2S_fichier_mkv}\" > \"${G2S_dossier_save}/${tags}.xml\""

			# Code d'encapsulage
			[[ ${encapsulage} == True ]] && mkvmerge+=" --global-tags \"${G2S_dossier_save}/${tags}.xml\"" ;;

		*)
			# Code pour extraction du fichier joint
			mkvextract_joint+=" ${valeurs[2]}:\"${G2S_dossier_save}/${valeurs[2]}_${valeurs[4]}\""

			# Ajout du fichier dans la liste des fichiers temps
			echo "${G2S_dossier_save}/${valeurs[2]}_${valeurs[4]}" >> "${tmp}/fichiers"

			# Code d'encapsulage
			[[ ${encapsulage} == True ]] && mkvmerge+=" --attachment-name \"${valeurs[4]}\" --attach-file \"${G2S_dossier_save}/${valeurs[2]}_${valeurs[4]}\"" ;;
	esac

	# Debug
	echo -e "-------Traitement de la piste ${track}-------
valeurs[0] : ${valeurs[0]}
valeurs[1] : ${valeurs[1]}
valeurs[2] : ${valeurs[2]}
valeurs[3] : ${valeurs[3]}
valeurs[4] : ${valeurs[4]}
valeurs[5] : ${valeurs[5]}
valeurs[6] : ${valeurs[6]}\n" >> "${log}"
done <<< "${tree_complet}"

# Debug
echo -e "<<<<<<<<<<<<<<<< Création des commandes post bouton extraction >>>>>>>>>>>>>>>>\n" >> "${log}"

# Si les variables mkvextract sont comme au debut de la fonction, on les reiniatilises
[[ ${mkvextract_track} == "mkvextract tracks \"${G2S_fichier_mkv}\"" ]] && unset mkvextract_track
[[ ${mkvextract_joint} == "mkvextract attachments \"${G2S_fichier_mkv}\"" ]] && unset mkvextract_joint

# Doit être lancé en fond sinon le bouton annulé ne fonctionne pas
actions &

# Creation du fichier indiquant qu'une action est en court en cas d'arret
echo "${!}" > "${tmp}/action_en_cours"
}


function actions
{
# Envoie à l'étape 5a
for cmd in mkvextract_track mkvextract_joint mkvextract_chap mkvextract_tag dts_ffmpeg convert_vob_srt mkvmerge
do
	# Verifie que la commande existe
	[[ -z ${!cmd} ]] && continue

	# Récupération de l'heure en seconde pour calculer le temps écoulé.
	temps_a=$(date +%s)

	# Execute la fonction en lien avec la commande
	case ${cmd} in
		mkvextract_track) extraction_piste ;;
		mkvextract_joint) extraction_fichier ;;
		mkvextract_chap) extraction_chapitre ;;
		mkvextract_tag) extraction_tag ;;
		dts_ffmpeg) dts ;;
		convert_vob_srt) convert_vob_srt ;;
		mkvmerge) encapsulage ;;
	esac

	# Destruction des variables
	unset pourcentage old_pourcentage

	# Récupération de l'heure en seconde pour calculer le temps écoulé.
	temps_b=$(date +%s)

	calcul_temps temps_a temps_b

	# Arret de la barre et envoie d'information et de debug
	case ${cmd} in
		mkvextract_track)
			bar_stop 1 "$(eval_gettext $'Track extraction completed in:') ${temps}" ;;

		mkvextract_joint)
			bar_stop 1 "$(eval_gettext $'Attached files extraction completed in:') ${temps} " ;;

		mkvextract_chap)
			echo "SET@_progressbar.set_text(''' $(eval_gettext $'Chapters file extraction completed in:') ${temps} ''')" ;;

		mkvextract_tag)
			echo "SET@_progressbar.set_text(''' $(eval_gettext $'Tags file extraction completed in:') ${temps} ''')" ;;

		dts_ffmpeg)
			bar_stop 1 "$(eval_gettext $'DTS conversion completed in:') ${temps}" ;;

		convert_vob_srt)
			bar_stop 1 "$(eval_gettext $'Vobsub conversion completed in:') ${temps}" ;;

		mkvmerge)
			bar_stop 1 "$(eval_gettext $'Tracks encapsulation completed in:') ${temps}" ;;
	esac
done

# Pause pour voir le texte ci-dessus
sleep 0.7

# Suppression des pistes temporaires si la variable contient des noms de fichiers...
if [[ ${encapsulage} == True || ${vob_srt} == True ]] && [[ ${G2S_fichier_tmp} == True ]]
then
	# Enleve le nom du mkv final de la liste des fichiers
	sed -i '1d' "${tmp}/fichiers"

	# Suppression des fichiers
	[[ -f "${tmp}/fichiers" ]] && while read; do [[ -f "${REPLY}" ]] && rm "${REPLY}"; done < "${tmp}/fichiers"

	# Debug
	echo -e "\n>>>>>>>>>>>>>>>> Liste des fichiers temporaires <<<<<<<<<<<<<<<<\n
$(< "${tmp}/fichiers")\n
<<<<<<<<<<<<<<<< Liste des fichiers temporaires >>>>>>>>>>>>>>>>" >> "${log}"
fi

# Récupération de l'heure en seconde pour calculer le temps écoulé.
temps_y=$(date +%s)

calcul_temps temps_x temps_y

# Pause pour laisser le temps de voir le temps de la derniere action realisée
sleep 1

echo "SET@_progressbar.set_text(''' $(eval_gettext $'Total operations time:') ${temps} ''')"

# Mise à 0 de la barre pour bien remettre au propre
echo 'SET@_progressbar.set_fraction(0)'
echo 'TREE@@HIZO@@liste_pistes'

terminal off
}


############################################
##### ETAPE 5a : EXTRACTION DES PISTES #####
############################################
# Code d'extraction des pistes
function extraction_piste
{
# creation du fichier d'extraction
echo "${mkvextract_track}" > "${tmp}/mkvextract_track"

# Debug
echo -e "\n>>>>>>>>>>>>>>>> Extraction des pistes <<<<<<<<<<<<<<<<\\n
${mkvextract_track}\\n
<<<<<<<<<<<<<<<< Extraction des pistes >>>>>>>>>>>>>>>>\n" >> "${log}"

# Lancement de la fonction gerant la barre de progression qui sera arrété dans la fonction actions
bar_pourcentage "$(eval_gettext $'Tracks extraction in progress...')"

# Code pour avancer la barre pendant le travail de mkvmextract
while read -d $'\r' pourcentage
do
	# Traitement d'affichage....
	pourcentage=${pourcentage##* } pourcentage=${pourcentage%?}

	echo "${pourcentage}" > "${tmp}/bar_pourcentage"
done < <(bash "${tmp}/mkvextract_track" 2>&1)
}


# Code d'extraction des fichiers joints
function extraction_fichier
{
# creation du fichier d'extraction
echo "${mkvextract_joint}" > "${tmp}/mkvextract_joint"

# Debug
echo -e "\n>>>>>>>>>>>>>>>> Extraction des fichiers joints <<<<<<<<<<<<<<<<\\n
${mkvextract_joint}\\n
<<<<<<<<<<<<<<<< Extraction des fichiers joints >>>>>>>>>>>>>>>>\n" >> "${log}"

# Code pour pulser la barre pendant le travail de mkvmextract qui sera arrété dans la fonction actions
bar_pulse "$(eval_gettext $'Attached files extraction in progress...')"

# Execution de la commande
bash "${tmp}/mkvextract_joint" 2>&1
}


# Code d'extraction des chapitres
function extraction_chapitre
{
# creation du fichier d'extraction
echo "${mkvextract_chap}" > "${tmp}/mkvextract_chap"

echo "SET@_progressbar.set_text(''' $(eval_gettext $'Chapters file extraction in progress...') ''')"

# Debug
echo -e "\n>>>>>>>>>>>>>>>> Extraction des chapitres <<<<<<<<<<<<<<<<\\n
${mkvextract_chap//>/→}\\n
<<<<<<<<<<<<<<<< Extraction des chapitres >>>>>>>>>>>>>>>>\n" >> "${log}"

bash "${tmp}/mkvextract_chap"
}


# Code d'extraction des tags
function extraction_tag
{
# creation du fichier d'extraction
echo "${mkvextract_tag}" > "${tmp}/mkvextract_tag"

echo "SET@_progressbar.set_text(''' $(eval_gettext $'Tags file extraction in progress...') ''')"

# Debug
echo -e "\n>>>>>>>>>>>>>>>> Extraction des tags <<<<<<<<<<<<<<<<\\n
${mkvextract_tag//>/→}\\n
<<<<<<<<<<<<<<<< Extraction des tags >>>>>>>>>>>>>>>>\n" >> "${log}"

bash "${tmp}/mkvextract_tag"
}


################################
##### ETAPE 5b : DTS > AC3 #####
################################
function dts
{
# creation du fichier d'extraction
echo "${dts_ffmpeg}" > "${tmp}/dts_ffmpeg"

# Récupération de la durée du film
time_film=$(sed -n "/Duration:/ s/.*Duration: \([0-9]*\).*/\1/p" "${tmp}/mkvinfo")

# Debug
echo -e "\n>>>>>>>>>>>>>>>> Conversion des DTS en AC3 <<<<<<<<<<<<<<<<\\n
${dts_ffmpeg}\\n
<<<<<<<<<<<<<<<< Conversion des DTS en AC3 >>>>>>>>>>>>>>>>\n" >> "${log}"

# Lancement de la fonction gerant la barre de progression qui sera arrété dans la fonction actions
bar_pourcentage "$(eval_gettext $'DTS conversion in progress...')"

# Code pour récupérer le retour de ffmpeg
while read -d $'\r' ligne
do
	if [[ "${ligne}" == *time=* ]]
	then
		# Recuperation du temps deja converti
		time=${ligne##*time=} time=${time%%.*}

		# Conversion de cet horaire en seconde
		time=$(date +%s -d "1970-01-01 ${time} seconds UTC")

		echo $(( (${time} * 100) / ${time_film} )) > "${tmp}/bar_pourcentage"
	fi
done < <(bash "${tmp}/dts_ffmpeg" 2>&1)
}


################################
##### ETAPE 5c : SUB > SRT #####
################################
function _sub_precedant
{
# Chargement des valeurs
source "${tmp}/sub_var"

# Sauvegarde du fichier précédant
echo "TEXT@@SAVE@@_sub_texte@@${sub_name}.txt"

# S'il y a des fichiers identiques, on copie les txt
[[ "${sub_name}" != "${sub_identiques[@]}" ]] && for file in "${sub_identiques[@]}"; do cp "${sub_name}.txt" "${file}.txt"; done

# Desincrémentation du num
((num--))

# Si dernier num, on bloque le bouton suivant
[[ ${num} -le 1 ]] && echo 'SET@_sub_precedant.set_sensitive(False)' || echo 'SET@_sub_precedant.set_sensitive(True)'
[[ ${num} -ge ${max_num} ]] && echo 'SET@_sub_suivant.set_sensitive(False)' || echo 'SET@_sub_suivant.set_sensitive(True)'

# Affichage de la progression
pourcentage=$(echo "scale=2;${num}/${max_num}" | bc)
echo "SET@_sub_progression.set_fraction(${pourcentage})"

# Chargement du fichier précédant
sub_identiques=$(sed -n "${num}p" "${tmp}/sub_list")
sub_name="${sub_identiques%% @@ *}"
sub_identiques="${sub_identiques#*@@ }"
sub_identiques=(${sub_identiques//@@})
echo "SET@_sub_image.set_from_file('${sub_name}')"

# Chargement du fichier s'il n'est pas vide (en cas de retour en arriere puis en avant)
[[ -s "${sub_name}" ]] && echo "TEXT@@LOAD@@_sub_texte@@${sub_name}.txt"

# Modification des variables
echo "num=${num}
max_num=${max_num}
sub_name=${sub_name}
sub_identiques=(${sub_identiques[@]})" > "${tmp}/sub_var"
}

function _sub_suivant
{
# Chargement des valeurs
source "${tmp}/sub_var"

# Sauvegarde du fichier précédant
echo "TEXT@@SAVE@@_sub_texte@@${sub_name}.txt"

# S'il y a des fichiers identiques, on copie les txt
[[ "${sub_name}" != "${sub_identiques[@]}" ]] && for file in "${sub_identiques[@]}"; do cp "${sub_name}.txt" "${file}.txt"; done

# Suppression du texte
echo "TEXT@@CLEAR@@_sub_texte"

# Incrémentation du num
((num++))

# Si dernier num, on bloque le bouton suivant
[[ ${num} -le 1 ]] && echo 'SET@_sub_precedant.set_sensitive(False)' || echo 'SET@_sub_precedant.set_sensitive(True)'
[[ ${num} -ge ${max_num} ]] && echo 'SET@_sub_suivant.set_sensitive(False)' || echo 'SET@_sub_suivant.set_sensitive(True)'

# Affichage de la progression
pourcentage=$(echo "scale=2;${num}/${max_num}" | bc)
echo "SET@_sub_progression.set_fraction(${pourcentage})"

# Chargement du fichier suivant
sub_identiques=$(sed -n "${num}p" "${tmp}/sub_list")
sub_name="${sub_identiques%% @@ *}"
sub_identiques="${sub_identiques#*@@ }"
sub_identiques=(${sub_identiques//@@})
echo "SET@_sub_image.set_from_file('${sub_name}')"

# Chargement du fichier s'il n'est pas vide (en cas de retour en arriere puis en avant)
[[ -s "${sub_name}" ]] && echo "TEXT@@LOAD@@_sub_texte@@${sub_name}.txt"

# Modification des variables
echo "num=${num}
max_num=${max_num}
sub_name=${sub_name}
sub_identiques=(${sub_identiques[@]})" > "${tmp}/sub_var"
}

function sub_termine
{
# Chargement des valeurs
source "${tmp}/sub_var"

# Sauvegarde du fichier précédant
echo "TEXT@@SAVE@@_sub_texte@@${sub_name}.txt"

# S'il y a des fichiers identiques, on copie les txt
[[ "${sub_name}" != "${sub_identiques[@]}" ]] && for file in "${sub_identiques[@]}"; do cp "${sub_name}.txt" "${file}.txt"; done

# Disparition de la fenetre
echo 'SET@_modif_sub.hide()'

# Fichier arretant le blocage
touch "${tmp}/sub_ok"
}

function convert_vob_srt
{
# Debug
echo -e "\n>>>>>>>>>>>>>>>> Conversion des vobsub en srt <<<<<<<<<<<<<<<<\n
$(< "${tmp}/vob_srt_a")\n
$(< "${tmp}/vob_srt_b")\n
$(< "${tmp}/vob_srt_c")\n
$(< "${tmp}/vob_srt_d")\n
<<<<<<<<<<<<<<<< Conversion des vobsub en srt >>>>>>>>>>>>>>>>\n" >> "${log}"

# Code pour pulser la barre pendant le travail qui sera arrété dans la fonction actions
bar_pourcentage "$(eval_gettext $'Vobsub conversion in progress... [take few minutes]')"

# Deplacement des fichiers
bash "${tmp}/vob_srt_a"

# Modification des couleurs pour être sur que ça sera lisible
sed -i "/^custom colors/ s/.*/custom colors: ON, tridx: 0000, colors: 000008, 300030, FFFFFF, 9332c8/" "${tmp}"/*.idx

# Creation des fichiers images
bash "${tmp}/vob_srt_b" 2>/dev/null

# Boucle calculant l'avancement
nb_trad_tot=$(ls -1 "${tmp}"/*.tif "${tmp}"/*.pgm 2>/dev/null | wc -l)
echo "0" > "${tmp}/bar_pourcentage"
while true
do
	# Nombre de fichier txt existant
	nb_trad=$(ls -1 "${tmp}"/*.txt 2>/dev/null | wc -l)

	# Calcul de l'avancement'
	pourcentage=$(echo "scale=1; ((${nb_trad} * 100) / ${nb_trad_tot})" | bc)

	# Envoie de l'avancement
	echo "${pourcentage}" > "${tmp}/bar_pourcentage"

	# Arrete la boucle si plus de fichier tif
	[[ ${nb_trad} == ${nb_trad_tot} || -f "${tmp}/bar_stop" ]] && break

	# Intervalle d'1' seconde entre les calculs
	sleep 1
done &

# Convertion en texte
bash "${tmp}/vob_srt_c" &>/dev/null

# Creation de la liste des fichiers non traduites en evitant les doublons
while read; do md5sum "${REPLY/%.txt}"; done < <(find "${tmp}" -name "*.txt" -empty) | sort | awk 'BEGIN{ORS=""}{if(t[$1]) sep=" @@ ";else {sep="";if(NR>1)print "\n"} t[$1]=1;print sep $2}END{print "\n"}'| sort > "${tmp}/sub_list"

if [[ -f "${tmp}/sub_list" && -s "${tmp}/sub_list" ]]
then
	num=1
	max_num=$(wc -l "${tmp}/sub_list" | cut -d' ' -f1)

	# Chargement du 1er fichier
	sub_identiques=$(sed -n "1p" "${tmp}/sub_list")
	sub_name="${sub_identiques%% @@ *}"
	sub_identiques="${sub_identiques#*@@ }"
	sub_identiques=(${sub_identiques//@@})
	echo "SET@_sub_image.set_from_file('${sub_name}')"

	# S'il n'y a qu'un seul fichier a traiter, on bloque le bouton suivant
	[[ ${num} == ${max_num} ]] && echo 'SET@_sub_suivant.set_sensitive("False")'

	# Affichage de la progression
	pourcentage=$(echo "scale=2;${num}/${max_num}" | bc)
	echo "SET@_sub_progression.set_fraction(${pourcentage})"

	# Sauvegarde des variables
	echo "num=${num}
max_num=${max_num}
sub_name=${sub_name}
sub_identiques=(${sub_identiques[@]})" > "${tmp}/sub_var"

	# Affichage de la fenetre
	echo 'SET@_modif_sub.show()' &

	# Blocage de la fonction
	while [[ ! -f "${tmp}/sub_ok" ]]; do sleep 0.5; done

	# Suppression du fichier bloquant
	rm "${tmp}/sub_ok"
fi

# Creation du fichier srt
bash "${tmp}/vob_srt_d" 2>/dev/null
}


#############################################
##### ETAPE 5d : ENCAPSULAGE DES PISTES #####
#############################################
function encapsulage
{
# creation du fichier d'extraction
echo "${mkvmerge}" > "${tmp}/mkvmerge"

# Ouverture des fichiers srt si option activée
if [[ ${G2S_open_sub} == True ]]
then
	while read
	do
		xdg-open "${REPLY}"

		# Blocage
		while [[ $(ps x | grep "${REPLY}" | grep -v "grep") ]]; do sleep 0.5; done
	done < <(sed 's/"/\n/g ; s/--/\n/g' "${tmp}/mkvmerge" | grep "\.srt")
fi

# Debug
echo -e "\n>>>>>>>>>>>>>>>> Encapsulage des pistes <<<<<<<<<<<<<<<<\n
${mkvmerge}" >> "${log}"

# Lancement de la fonction gerant la barre de progression qui sera arrété dans la fonction actions
bar_pourcentage "$(eval_gettext $'Tracks encapsulation in progress...')"

# Code pour récupérer le retour de mkvmerge
while read -d $'\r' ligne
do
	pourcentage=${ligne##* }

	echo "${pourcentage%?}" > "${tmp}/bar_pourcentage"

	if [[ ${ligne} =~ Error ]]
	then
		# Debug et info
		echo "SET@_progressbar.set_text(''' $(eval_gettext $'Error: mkvmerge to returned an error') ''')"
		echo -e "\Erreur : mkvmerge à renvoyé une erreur :${ligne}\n" >> "${log}"
		break
	fi
done < <(bash "${tmp}/mkvmerge" 2>&1)

# Debug
echo -e "\n<<<<<<<<<<<<<<<< Encapsulage des pistes >>>>>>>>>>>>>>>>\n" >> "${log}"
}


###################
##### Boutons #####
###################
function _bouton_quit
{
if [[ ${1} == clic_droit ]]
then
	reload=yes

	# Appuie graphiquement sur le bouton
	echo 'SET@_bouton_quit.activate()'

else
	# Sauvegarde des préférences
	echo 'CONFIG@@SAVE'

	# Fenetre cachée
	echo 'SET@_window1.hide()'; sleep 0.1

    # Gestion de l'adresse du fichier mkv
    if [[ -f "${G2S_fichier_mkv}" ]]
    then
        # Modification de la variable de l'adresse du fichier mkv dans le fichier de config pour conserver le dossier
        [[ ${G2S_auto_file} != True ]] && sed -i "/^_fichier_mkv/ s@= \(.*\)/.*\.mkv@= \1@" "${config}"

    elif [[ -d "${G2S_fichier_mkv}" ]]
    then
        # On conserve en mémoire le dossier
        sed -i "/^_fichier_mkv/ s@= .*@= ${G2S_fichier_mkv}@" "${config}"
    fi

	# Modification de la variable de l'adresse du fichier mkv dans le fichier de config pour conserver le dossier
	[[ ${G2S_auto_file} != True ]] && sed -i "/^_fichier_mkv/ s@= \(.*\)/.*\.mkv@= \1@" "${config}"

	# Modification de l'emplacement de la fenetre afin qu'elle soit toujours centrée
	sed -i "/^x \|^y / s/=.*/= -1/" "${config}"

	# Suppression du dossier temporaire s'il existe
	[[ -d "${tmp}" ]] && rm -r "${tmp}" &

	# Arret des actions en cours
	_bouton_stop

	# Arret ou reboot du logiciel
	[[ ${reload} == yes ]] && echo 'EXIT@@SAVE' || echo 'EXIT@@'
fi
}

function _mkvinfo { mkvinfo -g "${G2S_fichier_mkv}" & }

function _bouton_stop
{
# Si aucune action en cours, on arrete là
[[ ! -f "${tmp}/action_en_cours" ]] && return

# Récuération des sous pid
sous_pid_actions=$(ps --ppid $(< "${tmp}/action_en_cours") -o pid= 2>/dev/null)
sous_sous_pid_actions=$(ps --ppid ${sous_pid_actions} -o pid= 2>/dev/null)

# Suppression des fichiers
[[ -f "${tmp}/fichiers" ]] && while read; do [[ -f "${REPLY}" ]] && rm "${REPLY}"; done < "${tmp}/fichiers"

# Kill des pids
for pid in ${sous_sous_pid_actions} ${sous_pid_actions} $(< "${tmp}/action_en_cours")
do
	[[ $(ps h -p ${pid} 2>/dev/null) ]] && kill ${pid}
done

# Si c'est le bouton quit qui à appeller la fonction, on arrete la
[[ ${FUNCNAME[1]} == _bouton_quit ]] && return

# Arret de la barre
bar_stop 0 "$(eval_gettext $'Canceling and deleting files completed.')"

# Debug
echo -e "\n>>>>>>>>>>>>>>>> Bouton annuler <<<<<<<<<<<<<<<<\n
Annulation des travaux et suppression des fichiers :
$(< "${tmp}/fichiers")\n
<<<<<<<<<<<<<<<< Bouton annuler >>>>>>>>>>>>>>>>\n" >> "${log}"

# Remis en place des coches en relancant la fonction treeview
echo "TREE@@HIZO@@liste_pistes"

terminal off
}


##############################
##### Test et réparation #####
##############################
# Doit être lancé en fond sinon le bouton annulé ne fonctionne pas
function _verifier_mkv { _verifier_mkv_suite & echo "${!}" > "${tmp}/action_en_cours"; }

function _verifier_mkv_suite
{
# Récupération de l'heure en seconde pour calculer le temps écoulé.
temps_a=$(date +%s)

terminal on

# Mise en attente du script et pulsation de la barre de chargement
bar_pulse "$(eval_gettext $'MKV checking in progress...')"

# Debug
echo -e "\n>>>>>>>>>>>>>>>> Validation du fichier mkv <<<<<<<<<<<<<<<<\n
mkvalidator --no-warn \"${G2S_fichier_mkv}\"\n
<<<<<<<<<<<<<<<< Validation du fichier mkv >>>>>>>>>>>>>>>>\n" >> "${log}"

# lancement de la commande
mkvalidator --no-warn "${G2S_fichier_mkv}"

# Récupération de l'heure en seconde pour calculer le temps écoulé.
temps_b=$(date +%s)

calcul_temps temps_a temps_b

echo "SET@_progressbar.set_text(''' $(eval_gettext $'MKV checking completed in:') ${temps} ''')"

# Arret de la barre
bar_stop 1

# Laisse le temps de lire
sleep 2

terminal off
}

# Doit être lancé en fond sinon le bouton annulé ne fonctionne pas
function _reparer_mkv { _reparer_mkv_suite & echo "${!}" > "${tmp}/action_en_cours"; }

function _reparer_mkv_suite
{
# Récupération de l'heure en seconde pour calculer le temps écoulé.
temps_a=$(date +%s)

terminal on

# Modifications graphiques
[[ ${G2S_auto_save} == True ]] && echo "SET@_dossier_save_tmp.set_current_folder('${G2S_fichier_mkv%/*}')"
echo 'SET@_progressbar.set_fraction(0)'

# Mise en attente du script et pulsation de la barre de chargement
bar_pulse "$(eval_gettext $'MKV repairing in progress...')"

# Debug
echo -e "\n>>>>>>>>>>>>>>>> Correction du fichier mkv <<<<<<<<<<<<<<<<\n
mkclean --optimize \"${G2S_fichier_mkv}\" \"${G2S_dossier_save}/mkclean-${nom_fichier}\"\n
<<<<<<<<<<<<<<<< Correction du fichier mkv >>>>>>>>>>>>>>>>\n" >> "${log}"

# Récupération du nom du fichier en cas d'annulation'
echo "${G2S_dossier_save}/mkclean-${nom_fichier}" > "${tmp}/fichiers"

# Lancement de la commande
mkclean --quiet --optimize "${G2S_fichier_mkv}" "${G2S_dossier_save}/mkclean-${nom_fichier}"

# Récupération de l'heure en seconde pour calculer le temps écoulé.
temps_b=$(date +%s)

calcul_temps temps_a temps_b

echo "SET@_progressbar.set_text(''' $(eval_gettext $'MKV repairing completed in:') ${temps} ''')"

# Arret de la barre
bar_stop 1

# Laisse le temps de lire
sleep 2

terminal off
}


#######################
##### Préférences #####
#######################
function open_log { xdg-open "${log}"; }

function del_pref
{
# Création d'un nouveau fichier de config
echo "[TOGGLE]
_canaux = False
_fichier_tmp = True
_debug = False
_auto_save = False
_auto_file = False
_remplacer = False
_open_sub = False

[COMBO]
_bitrate = 0
_langue = 1
_boost = 0

[WINDOW:_window1]
width = -1
height = -1
x = -1
y = -1

[FILECHOOSER]
_fichier_mkv = ${HOME}
_dossier_save = ${HOME}" > "${config}"

# Récupération des variables
echo "CONFIG@@GET@@@@@@${config}"; sleep 0.1

# Chargement graphique de ce nouveau fichier
echo "CONFIG@@LOAD@@@@@@${config}"
}


########################
##### Introduction #####
########################
### Variables
mkv_extractor_gui="${HOME}/.config/mkv-extractor-gui"
config="${mkv_extractor_gui}/config.cfg"
log="${mkv_extractor_gui}/mkv-extractor-gui.log"
tmp=$(mktemp -d /tmp/mkv-extractor-gui.XXX)
version='4.9.1'
flag_scale=False

# Variables en fonction de la langue
tags="$(eval_gettext $'Tags')"
chapters="$(eval_gettext $'Chapters')"

# Gestion du fichier log enregistrant tout probleme
[[ -f "${log}" ]] && rm "${log}"
exec 2>> "${log}"

[[ $(type -p mkvalidator) ]] && mkvalidator=True || mkvalidator=False
[[ $(type -p mkclean) ]] && mkclean=True || mkclean=False
[[ $(type -p ffmpeg) ]] && ffmpeg=True || ffmpeg=False

# Modification graphique
echo 'FILE@@SETFILTER@@_fichier_mkv_tmp@@add_mime_type@@video/x-matroska'
echo "SET@_window1.set_title('MKV Extractor Gui v${version} - by Hizoka')"
echo "SET@MKV_Extractor_Gui.set_version('v${version}')"

# Validation des adresses
if [[ -d "${G2S_dossier_save}" ]]
then
    echo "SET@_dossier_save.set_icon_from_stock(1, gtk.STOCK_YES)"
    echo "SET@_dossier_save_tmp.set_filename('''${G2S_dossier_save}''')"
fi

if [[ -f "${G2S_fichier_mkv}" ]]
then
    # Dans le cas ou un fichier est renseigné (clic droit ouvrir avec)
    echo "SET@_fichier_mkv.set_icon_from_stock(1, gtk.STOCK_YES)"
    echo "SET@_fichier_mkv_tmp.set_filename('''${G2S_fichier_mkv}''')"

elif [[ -d "${G2S_fichier_mkv}" ]]
then
    # Dans le cas ou le dossier est conservé en mémoire
    echo 'SET@_fichier_mkv.set_text("")'
    echo "SET@_fichier_mkv_tmp.set_current_folder('''${G2S_fichier_mkv}''')"

else
    # Dans le cas ou il n'y a rien ou que ce n'est pas valide
    echo 'SET@_fichier_mkv.set_text("")'
fi

# Affichage de la fenetre
echo 'SET@_window1.show()'

echo -e ">>>>>>>>>>>>>>>> Informations globales <<<<<<<<<<<<<<<<\n
Version de Mkvmerge : $(mkvmerge -V | sed 's/.*v// ; s/ .*//')
Version de MKV Extractor Gui : ${version}
Version d'Ubuntu : $(uname -m)
$(< /etc/lsb-release 2>/dev/null)
Fichier ouvert directement : ${G2S_fichier_mkv}\n
<<<<<<<<<<<<<<<< Informations globales >>>>>>>>>>>>>>>>\n" >> "${log}"


#######################################
##### Chargement  des préférences #####
#######################################
# Chargement des valeurs des combobox
case ${G2S_bitrate} in
	0) G2S_bitrate=128 ;;
	1) G2S_bitrate=192 ;;
	2) G2S_bitrate=224 ;;
	3) G2S_bitrate=256 ;;
	4) G2S_bitrate=320 ;;
	5) G2S_bitrate=384 ;;
	6) G2S_bitrate=448 ;;
	7) G2S_bitrate=512 ;;
	8) G2S_bitrate=576 ;;
	9) G2S_bitrate=640 ;;
esac

case ${G2S_boost} in
	0) G2S_boost=0 ;;
	1) G2S_boost=2 ;;
	2) G2S_boost=3 ;;
	3) G2S_boost=4 ;;
	4) G2S_boost=5 ;;
esac


# Charge la video si elle existe
[[ -f "${G2S_fichier_mkv}" ]] && _fichier_mkv "${G2S_fichier_mkv}"


######################################################################################################

while read ligne
do
	if [[ ${ligne} =~ GET@ ]]
	then
		eval ${ligne#*@}
		echo "DEBUG => in boucle bash :" ${ligne#*@}
	else
		echo "DEBUG=> in bash NOT GET" ${ligne}
		${ligne}
	fi
done < <(while true
do
	read entree < ${FIFO}
	[[ ${entree}  == QuitNow ]] && break
	echo ${entree}
done)

[[ -e /tmp/FIFO${PID} ]] && rm /tmp/FIFO${PID}

exit
